"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.randomizeCharacters = exports.generateCharacters = exports.generatePin = exports.generatePassword = exports.SYMBOL_CHARSET = exports.UPPERCASE_CHARSET = exports.LOWERCASE_CHARSET = exports.DIGIT_CHARSET = void 0;
const crypto_1 = require("./crypto");
exports.DIGIT_CHARSET = '0123456789';
exports.LOWERCASE_CHARSET = 'abcdefghijklmnopqrstuvwxyz';
exports.UPPERCASE_CHARSET = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
exports.SYMBOL_CHARSET = '!"#$%&\'()*+,-./:;<=>?@[]{}^_`|~';
function generatePassword(options) {
    const defaults = {
        length: 12,
        digits: true,
        symbols: true,
        uppercase: true,
        lowercase: true,
    };
    options = options || {};
    return createPassword({
        length: options.length === undefined ? defaults.length : options.length,
        digits: options.digits === undefined ? defaults.digits : options.digits,
        symbols: options.symbols === undefined ? defaults.symbols : options.symbols,
        lowercase: options.lowercase === undefined ? defaults.lowercase : options.lowercase,
        uppercase: options.uppercase === undefined ? defaults.uppercase : options.uppercase,
    });
}
exports.generatePassword = generatePassword;
function createPassword(options) {
    validatePasswordOptions(options);
    const passwordLength = options.length;
    const [initDigitLength, moreDigits] = getLengthForOption(options.digits);
    const [initSymbolLength, moreSymbols] = getLengthForOption(options.symbols);
    const [initLowercaseLength, moreLowercase] = getLengthForOption(options.lowercase);
    const [initUppercaseLength, moreUppercase] = getLengthForOption(options.uppercase);
    let result = generateCharacters(initDigitLength, exports.DIGIT_CHARSET) +
        generateCharacters(initSymbolLength, exports.SYMBOL_CHARSET) +
        generateCharacters(initLowercaseLength, exports.LOWERCASE_CHARSET) +
        generateCharacters(initUppercaseLength, exports.UPPERCASE_CHARSET);
    let remainingCharset = '';
    if (moreDigits) {
        remainingCharset += exports.DIGIT_CHARSET;
    }
    if (moreSymbols) {
        remainingCharset += exports.SYMBOL_CHARSET;
    }
    if (moreLowercase) {
        remainingCharset += exports.LOWERCASE_CHARSET;
    }
    if (moreUppercase) {
        remainingCharset += exports.UPPERCASE_CHARSET;
    }
    result += generateCharacters(passwordLength - result.length, remainingCharset);
    return randomizeCharacters(result);
}
function validatePasswordOptions(options) {
    const { length } = options;
    if (typeof length !== 'number' || length < 1) {
        throw new Error('Invalid option: length option must be a number greater than or equal to 1');
    }
    const [initDigitLength, moreDigits] = getLengthForOption(options.digits);
    if (typeof initDigitLength !== 'number' || initDigitLength < 0) {
        throw new Error('Invalid option: digits option must be a boolean, number, or object');
    }
    const [initSymbolLength, moreSymbols] = getLengthForOption(options.symbols);
    if (typeof initSymbolLength !== 'number' || initSymbolLength < 0) {
        throw new Error('Invalid option: symbols option must be a boolean, number, or object');
    }
    const [initLowercaseLength, moreLowercase] = getLengthForOption(options.lowercase);
    if (typeof initLowercaseLength !== 'number' || initLowercaseLength < 0) {
        throw new Error('Invalid option: lowercase option must be a boolean, number, or object');
    }
    const [initUppercaseLength, moreUppercase] = getLengthForOption(options.uppercase);
    if (typeof initUppercaseLength !== 'number' || initUppercaseLength < 0) {
        throw new Error('Invalid option: uppercase option must be a boolean, number, or object');
    }
    const sum = initDigitLength + initSymbolLength + initLowercaseLength + initUppercaseLength;
    const allExact = !moreDigits && !moreSymbols && !moreLowercase && !moreUppercase;
    if (sum > length) {
        throw new Error('Invalid option: Requested characters exceeds expected length');
    }
    if (allExact && sum !== length) {
        throw new Error('Invalid option: Requested less characters than expected length');
    }
}
function getLengthForOption(option) {
    switch (typeof option) {
        case 'boolean':
            return [0, option];
        case 'number':
            return [option, false];
        case 'object':
            return [option.min, true];
        default:
            return [undefined, undefined];
    }
}
function generatePin(length) {
    if (typeof length !== 'number' || length < 1) {
        throw new Error('Invalid argument: length argument must be a number greater than or equal to 1');
    }
    return generateCharacters(length, exports.DIGIT_CHARSET);
}
exports.generatePin = generatePin;
function generateCharacters(length, charset) {
    if (typeof length !== 'number' || length < 0) {
        throw new Error('Invalid argument: length argument must be a number greater than or equal to 0');
    }
    if (typeof charset !== 'string' || charset.length < 2) {
        throw new Error('Invalid argument: charset argument must be a string with length greater than or equal to 2');
    }
    return (0, crypto_1.getRandomValues)(length, charset.length).reduce((characters, i) => {
        return characters + charset[i];
    }, '');
}
exports.generateCharacters = generateCharacters;
function randomizeCharacters(characters) {
    if (typeof characters !== 'string') {
        throw new Error('Invalid argument: characters argument must be a string');
    }
    const charactersLength = characters.length;
    const swapIndices = (0, crypto_1.getRandomValues)(charactersLength, charactersLength);
    const result = Array.from(characters);
    for (let i = 0; i < charactersLength; i++) {
        const j = swapIndices[i];
        const temp = result[i];
        result[i] = result[j];
        result[j] = temp;
    }
    return result.join('');
}
exports.randomizeCharacters = randomizeCharacters;
